@@ -228,6 +228,11 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
 
   vc <- nrow(adjmatrix)
 
+  ## to remove non-redundancies that can persist in a dgtMatrix
+  if(is(adjmatrix, "dgTMatrix")) {
+    adjmatrix = as(adjmatrix, "CsparseMatrix")
+  }
+  
   if (is.null(weighted) && mode=="undirected") { mode <- "max" }
   
   if (mode == "directed") {
@@ -646,3 +651,180 @@ graph.famous <- function(name) {
   .Call("R_igraph_famous", as.character(name),
         PACKAGE="igraph")
 }
+
+graph.full.bipartite <- function(n1, n2, directed=FALSE,
+                                 mode=c("all", "out", "in")) {
+
+  n1 <- as.numeric(n1)
+  n2 <- as.numeric(n2)
+  directed <- as.logical(directed)
+  mode <- switch(igraph.match.arg(mode), "out"=1, "in"=2, "all"=3, "total"=3)  
+  
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
+  res <- .Call("R_igraph_full_bipartite", n1, n2, as.logical(directed), mode,
+               PACKAGE="igraph")
+  set.vertex.attribute(res$graph, "type", value=res$types)
+}
+
+graph.bipartite <- function(types, edges, directed=FALSE) {
+
+  types <- as.logical(types)
+  edges <- as.numeric(edges)
+  directed <- as.logical(directed)
+
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
+  res <- .Call("R_igraph_create_bipartite", types, edges, directed,
+               PACKAGE="igraph")
+  set.vertex.attribute(res, "type", value=types)
+}
+
+graph.incidence.sparse <- function(incidence, directed, mode, multiple,
+                                   weighted) {
+  require(Matrix)
+
+  n1 <- nrow(incidence)
+  n2 <- ncol(incidence)
+  el <- selectMethod("summary", signature=c(object="sparseMatrix"))(incidence)
+  ## el <- summary(incidence)
+  el[,2] <- el[,2] + n1
+  el[,c(1,2)] <- el[,c(1,2)] - 1
+
+  if (!is.null(weighted)) {
+
+    if (is.logical(weighted) && weighted) {
+      weighted <- "weight"
+    }
+    if (!is.character(weighted)) {
+      stop("invalid value supplied for `weighted' argument, please see docs.")
+    }
+
+    if (!directed || mode==1) {
+      ## nothing do to
+    } else if (mode==2) {
+      el[,1:2] <- el[,c(2,1)]
+    } else if (mode==3) {
+      el <- rbind(el, el[,c(2,1,3)])
+    }
+
+    res <- graph.empty(n=n1+n2, directed=directed)
+    weight <- list(el[,3])
+    names(weight) <- weighted
+    res <- add.edges(res, edges=t(as.matrix(el[,1:2])), attr=weight)
+
+  } else {
+
+    if (multiple) {
+      el[,3] <- ceiling(el[,3])
+      el[,3][ el[,3] < 0 ] <- 0
+    } else {
+      el[,3] <- el[,3] != 0
+    }
+
+    edges <- unlist(apply(el, 1, function(x) rep(unname(x[1:2]), x[3])))
+    res <- graph(n=n1+n2, edges, directed=directed)
+  } 
+    
+  set.vertex.attribute(res, "type", value=c(rep(FALSE, n1), rep(TRUE, n2)))
+}
+
+graph.incidence.dense <- function(incidence, directed, mode, multiple,
+                                  weighted) {
+  
+  if (!is.null(weighted)) {
+    if (is.logical(weighted) && weighted) {
+      weighted <- "weight"
+    }
+    if (!is.character(weighted)) {
+      stop("invalid value supplied for `weighted' argument, please see docs.")
+    }
+
+    n1 <- nrow(incidence)
+    n2 <- ncol(incidence)
+    no.edges <- sum(incidence != 0)
+    if (directed && mode==3) { no.edges <- no.edges * 2 }
+    edges <- numeric(2*no.edges)
+    weight <- numeric(no.edges)
+    ptr <- 1
+    for (i in seq_len(nrow(incidence))) {
+      for (j in seq_len(ncol(incidence))) {
+        if (incidence[i,j] != 0) {
+          if (!directed || mode==1) {
+            edges[2*ptr-1] <- i-1
+            edges[2*ptr] <- n1+j-1
+            weight[ptr] <- incidence[i,j]
+            ptr <- ptr + 1
+          } else if (mode==2) {
+            edges[2*ptr-1] <- n1+j-1
+            edges[2*ptr] <- i-1
+            weight[ptr] <- incidence[i,j]
+            ptr <- ptr + 1
+          } else if (mode==3) {
+            edges[2*ptr-1] <- i-1
+            edges[2*ptr] <- n1+j-1
+            weight[ptr] <- incidence[i,j]
+            ptr <- ptr + 1
+            edges[2*ptr-1] <- n1+j-1
+            edges[2*ptr] <- i-1
+          }
+        }
+      }
+    }
+    res <- graph.empty(n=n1+n2, directed=directed)
+    weight <- list(weight)
+    names(weight) <- weighted
+    res <- add.edges(res, edges, attr=weight)
+    res <- set.vertex.attribute(res, "type",
+                                value=c(rep(FALSE, n1), rep(TRUE, n2)))
+    
+  } else {
+
+    mode(incidence) <- "double"
+    on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
+    ## Function call
+    res <- .Call("R_igraph_incidence", incidence, mode, directed, multiple,
+                 PACKAGE="igraph")
+    res <- set.vertex.attribute(res$graph, "type", value=res$types)
+
+  }
+
+  res
+}
+
+graph.incidence <- function(incidence, directed=FALSE,
+                            mode=c("all", "out", "in", "total"), 
+                            multiple=FALSE, weighted=NULL,
+                            add.names=NULL) {
+  # Argument checks
+  directed <- as.logical(directed)
+  mode <- switch(igraph.match.arg(mode), "out"=1, "in"=2, "all"=3, "total"=3)
+  multiple <- as.logical(multiple)
+
+  if (is(incidence, "Matrix")) {
+    res <- graph.incidence.sparse(incidence, directed=directed,
+                                  mode=mode, multiple=multiple,
+                                  weighted=weighted)
+  } else {
+    res <- graph.incidence.dense(incidence, directed=directed, mode=mode,
+                                 multiple=multiple, weighted=weighted)
+  }
+
+  ## Add names
+  if (is.null(add.names)) {
+    if (!is.null(rownames(incidence)) && !is.null(colnames(incidence))) {
+      add.names <- "name"
+    } else {
+      add.names <- NA
+    }
+  } else if (!is.na(add.names)) {
+    if (is.null(rownames(incidence)) || is.null(colnames(incidence))) {
+      warning("Cannot add row- and column names, at least one of them is missing")
+      add.names <- NA
+    }
+  }
+  if (!is.na(add.names)) {
+    res <- set.vertex.attribute(res, add.names,
+                                value=c(rownames(incidence), colnames(incidence)))
+  }
+  res
+}
+
