@@ -23,6 +23,7 @@
 
 #include "igraph.h"
 #include "config.h"
+#include "igraph_math.h"
 #include "gml_tree.h"
 #include "memory.h"
 
@@ -1544,7 +1545,7 @@ int igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
       igraph_strvector_get(&nvec, to, &str2);
       ret=fprintf(outstream, "%s %s\n", str1, str2);
       if (ret<0) {
-	IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+        IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       }
       IGRAPH_EIT_NEXT(it);
     }
@@ -1552,36 +1553,33 @@ int igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
     IGRAPH_FINALLY_CLEAN(1);
   } else if (names==0) {
     /* No names but weights */
-    igraph_strvector_t wvec;
-    IGRAPH_CHECK(igraph_strvector_init(&wvec, igraph_ecount(graph)));
-    IGRAPH_FINALLY(igraph_strvector_destroy, &wvec);
-    IGRAPH_CHECK(igraph_i_attribute_get_string_edge_attr(graph, weights, 
+    igraph_vector_t wvec;
+    IGRAPH_VECTOR_INIT_FINALLY(&wvec, igraph_ecount(graph));
+    IGRAPH_CHECK(igraph_i_attribute_get_numeric_edge_attr(graph, weights, 
 							 igraph_ess_all(IGRAPH_EDGEORDER_FROM), 
 							 &wvec));
     while (!IGRAPH_EIT_END(it)) {
       igraph_integer_t edge=IGRAPH_EIT_GET(it);
       igraph_integer_t from, to;
       int ret=0;
-      char *str1;
       igraph_edge(graph, edge, &from, &to);
-      igraph_strvector_get(&wvec, edge, &str1);
-      ret=fprintf(outstream, "%li %li %s\n", 
-		  (long int)from, (long int)to, str1);
+      ret=fprintf(outstream, "%li %li %f\n", 
+		  (long int)from, (long int)to, VECTOR(wvec)[(long int)edge]);
       if (ret<0) {
-	IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+        IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       }
       IGRAPH_EIT_NEXT(it);      
     }
-    igraph_strvector_destroy(&wvec);
+    igraph_vector_destroy(&wvec);
     IGRAPH_FINALLY_CLEAN(1);
   } else {
     /* Both names and weights */
-    igraph_strvector_t nvec, wvec;
-    IGRAPH_CHECK(igraph_strvector_init(&wvec, igraph_ecount(graph)));
-    IGRAPH_FINALLY(igraph_strvector_destroy, &wvec);
+    igraph_strvector_t nvec;
+	igraph_vector_t wvec;
+    IGRAPH_VECTOR_INIT_FINALLY(&wvec, igraph_ecount(graph));
     IGRAPH_CHECK(igraph_strvector_init(&nvec, igraph_vcount(graph)));
     IGRAPH_FINALLY(igraph_strvector_destroy, &nvec);
-    IGRAPH_CHECK(igraph_i_attribute_get_string_edge_attr(graph, weights, 
+    IGRAPH_CHECK(igraph_i_attribute_get_numeric_edge_attr(graph, weights, 
 							 igraph_ess_all(IGRAPH_EDGEORDER_FROM), 
 							 &wvec));
     IGRAPH_CHECK(igraph_i_attribute_get_string_vertex_attr(graph, names, 
@@ -1591,23 +1589,22 @@ int igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
       igraph_integer_t edge=IGRAPH_EIT_GET(it);
       igraph_integer_t from, to;
       int ret=0;
-      char *str1, *str2, *str3;
+      char *str1, *str2;
       igraph_edge(graph, edge, &from, &to);
       igraph_strvector_get(&nvec, from, &str1);
       igraph_strvector_get(&nvec, to, &str2);
-      igraph_strvector_get(&wvec, edge, &str3);
       ret=fprintf(outstream, "%s %s ", str1, str2);
       if (ret<0) {
-	IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+        IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       }
-      ret=fprintf(outstream, "%s\n", str3);
+      ret=fprintf(outstream, "%f\n", VECTOR(wvec)[(long int)edge]);
       if (ret<0) {
-	IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+        IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       }
       IGRAPH_EIT_NEXT(it);
     }
     igraph_strvector_destroy(&nvec);
-    igraph_strvector_destroy(&wvec);
+    igraph_vector_destroy(&wvec);
     IGRAPH_FINALLY_CLEAN(2);
   }
   
