@@ -25,6 +25,7 @@
 #include "memory.h"
 #include "random.h"
 #include "error.h"
+#include "config.h"
 
 #include <assert.h>
 #include <string.h> 		/* memcpy & co. */
@@ -263,6 +264,17 @@ int igraph_vector_ptr_push_back (igraph_vector_ptr_t* v, void* e) {
 	return 0;
 }
 
+void *igraph_vector_ptr_pop_back (igraph_vector_ptr_t *v) {
+	void *tmp;
+	assert(v != NULL);
+	assert(v->stor_begin != NULL);
+	assert(v->stor_begin != v->end);
+	tmp=*(v->end);
+	v->end -= 1;
+	  
+	return 0;
+}
+
 /**
  * \ingroup vectorptr
  * \function igraph_vector_ptr_insert
